package com.mcwl.web.controller.resource;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.mcwl.common.core.controller.BaseController;
import com.mcwl.common.core.domain.AjaxResult;
import com.mcwl.common.core.page.TableDataInfo;
import com.mcwl.common.utils.SecurityUtils;
import com.mcwl.resource.domain.ModelProduct;
import com.mcwl.resource.domain.dto.ModelImagePageRes;
import com.mcwl.resource.domain.request.RequestModel;
import com.mcwl.resource.domain.request.RequestWorkFlow;
import com.mcwl.resource.domain.vo.MallProductVo;
import com.mcwl.resource.service.ModelService;
import com.mcwl.system.service.ISysUserService;
import com.mcwl.web.controller.common.OssUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;


/**
 * 商品
 * @Author：ChenYan
 * @Project：McWl
 * @Package：com.mcwl.web.controller.resource
 * @Filename：MallProductController
 * @Description 商品
 * @Date：2024/12/31 10:48
 */

@RestController
@RequestMapping("/model")
public class MallProductController extends BaseController {
    @Autowired
    private  ISysUserService sysUserService;

    @Autowired
    private ModelService modelService;


    /***
     *
     * 图片
     * @param file
     * @return
     */
    @PostMapping("/file")
    public AjaxResult Malifile(@RequestParam MultipartFile file){

        String s = OssUtil.uploadMultipartFile(file);
        return AjaxResult.success(s);
    }


    /***
     *
     * zip
     * @param file
     * @return
     */
    @PostMapping("/zipUrlFile")
    public AjaxResult zipUrlFile(@RequestParam MultipartFile file){
        String s = OssUtil.uploadMultipartFile(file);
        return AjaxResult.success(s);
    }




    /***
     *
     * 下载zip
     * @param file
     * @return
     */
    @PostMapping("/zipUrl")
    public AjaxResult zipUrl(@RequestParam MultipartFile file){
        String s = OssUtil.uploadMultipartFile(file);
        return AjaxResult.success(s);
    }





    /**
     * 模型列表
     */
    @PostMapping("/list")
    public TableDataInfo list(@RequestBody ModelImagePageRes imagePageRes) {

        return modelService.listByPage(imagePageRes);
    }




    @PostMapping("finbyid")
    public AjaxResult finbyid(@RequestBody ModelProduct modelVersion)
    {
        ModelProduct modelVersion1 = modelService.getById(modelVersion.getId());
        return AjaxResult.success(modelVersion1);
    }


    @PostMapping("insert")
    public AjaxResult insert(@RequestBody ModelProduct modelVersion)
    {
        Long userId = SecurityUtils.getUserId();
        modelVersion.setUserId(userId);
        modelVersion.setCreateBy(getUsername());
        modelService.save(modelVersion);
        return AjaxResult.success();
    }

    @PostMapping("update")
    public AjaxResult update(@RequestBody ModelProduct modelVersion)
    {
        Long userId = SecurityUtils.getUserId();
        modelVersion.setUserId(userId);
        modelVersion.setUpdateBy(getUsername());
        modelService.updateById(modelVersion);
        return AjaxResult.success();
    }




    @PostMapping("/insert")
    public AjaxResult addupdateModel(@RequestBody RequestModel requestModel){

        return modelService.addModel(requestModel);
    }

    @PostMapping("/update")
    public AjaxResult updateModel(@RequestBody RequestModel requestModel){

        modelService.updaModel(requestModel);

        return AjaxResult.success("修改成功");
    }


    @PostMapping("delete")
    public AjaxResult delete(@RequestBody ModelProduct modelVersion)
    {
        modelService.removeById(modelVersion.getId());
        return AjaxResult.success();
    }

    /**
     * 查看用户发布的作品
     * @return
     */
    @PostMapping("/selectByUserId")
    public AjaxResult selectByUserId(@RequestBody MallProductVo mallProductVo){

        Page<ModelProduct> mallProductList = modelService.selectByUserId(mallProductVo);
        return AjaxResult.success(mallProductList);
    }

}
