package com.mcwl.web.controller.resource;

import cn.hutool.core.bean.BeanUtil;
import com.mcwl.common.core.domain.AjaxResult;
import com.mcwl.common.core.domain.entity.SysUser;
import com.mcwl.common.core.page.TableDataInfo;
import com.mcwl.common.utils.SecurityUtils;
import com.mcwl.common.utils.oss.OssUtil;
import com.mcwl.resource.domain.ModelImage;
import com.mcwl.resource.domain.dto.ModelImageCommentRes;
import com.mcwl.resource.domain.dto.ModelImagePageRes;
import com.mcwl.resource.domain.dto.ModelImageRes;
import com.mcwl.resource.domain.vo.ModelImageCommentVo;
import com.mcwl.resource.domain.vo.ModelImageVo;
import com.mcwl.resource.service.ModelImageCommentLikeService;
import com.mcwl.resource.service.ModelImageCommentService;
import com.mcwl.resource.service.ModelImageLikeService;
import com.mcwl.resource.service.ModelImageService;
import com.mcwl.system.service.ISysUserService;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.validation.constraints.NotNull;
import java.util.List;
import java.util.Objects;

/**
 * 图片
 */
@RestController
@RequestMapping("/modelImage")
@RequiredArgsConstructor
public class ModelImageController {

    private final ModelImageService modelImageService;

    private final ModelImageLikeService modelImageLikeService;

    private final ModelImageCommentLikeService modelImageCommentLikeService;

    private final ModelImageCommentService modelImageCommentService;

    private final ISysUserService sysUserService;


    /**
     * 图片列表
     */
    @PostMapping("/list")
    public TableDataInfo list(@RequestBody ModelImagePageRes imagePageRes) {

        return modelImageService.listByPage(imagePageRes);
    }

    /**
     * 图片详情
     */
    @GetMapping("/detail/{imageId}")
    public AjaxResult detail(@PathVariable @NotNull(message = "图片id不能为空") Long imageId) {
        ModelImageVo modelImageVo = new ModelImageVo();
        ModelImage modelImage = modelImageService.getById(imageId);
        if (Objects.nonNull(modelImage)) {
            BeanUtil.copyProperties(modelImage, modelImageVo);
            SysUser sysUser = sysUserService.selectUserById(modelImage.getUserId());
            modelImageVo.setUserId(SecurityUtils.getUserId());
            modelImageVo.setUserName(SecurityUtils.getUsername());
            modelImageVo.setUserAvatar(sysUser.getAvatar());
        }
        return AjaxResult.success(modelImageVo);
    }

    /**
     * 图片删除
     */
    @GetMapping("/delete/{imageId}")
    public AjaxResult delete(@PathVariable @NotNull(message = "图片id不能为空") Long imageId) {
        modelImageService.removeById(imageId);
        return AjaxResult.success();
    }

    /**
     * 图片修改
     */
    @PostMapping("/update")
    public AjaxResult update(@RequestBody ModelImageRes modelImageRes) {
        modelImageService.updateById(modelImageRes);
        return AjaxResult.success();
    }


    /**
     * 图片上传
     */
    @GetMapping("/upload")
    public AjaxResult upload(MultipartFile file) {

        return AjaxResult.success("上传成功", OssUtil.uploadMultipartFile(file));

    }


    /**
     * 图片发布
     */
    @PostMapping("/publish")
    public AjaxResult publish(@RequestBody ModelImageRes modelImageRes) {
        modelImageService.publish(modelImageRes);
        return AjaxResult.success();
    }

    /**
     * 图片点赞/取消
     */
    @GetMapping("/imageLike/{imageId}")
    public AjaxResult like(@PathVariable @NotNull(message = "图片id不能为空") Long imageId) {
        modelImageLikeService.like(imageId);
        return AjaxResult.success();
    }


    /**
     * 图片评论发布
     */
    @PostMapping("/comment")
    public AjaxResult comment(@RequestBody ModelImageCommentRes modelImageCommentRes) {
        modelImageService.comment(modelImageCommentRes);
        return AjaxResult.success();
    }

    /**
     * 图片评论点赞/取消
     */
    @GetMapping("/commentLike/{commentId}")
    public AjaxResult commentLike(@PathVariable @NotNull(message = "评论id不能为空") Long commentId) {
        modelImageCommentLikeService.like(commentId);
        return AjaxResult.success();
    }

    /**
     * 删除图片评论
     */
    @GetMapping("/commentDelete/{commentId}")
    public AjaxResult commentDelete(@PathVariable @NotNull(message = "评论id不能为空") Long commentId) {
        modelImageCommentService.removeById(commentId);
        return AjaxResult.success();
    }

    /**
     * 获取图片评论
     */
    @GetMapping("/comment/{imageId}")
    public AjaxResult getComment(@PathVariable @NotNull(message = "图片id不能为空") Long imageId) {
        List<ModelImageCommentVo> modelImageCommentVoList = modelImageService.getComment(imageId);
        return AjaxResult.success(modelImageCommentVoList);
    }


}
