package com.mcwl.resource.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.mcwl.common.constant.HttpStatus;
import com.mcwl.common.core.domain.AjaxResult;
import com.mcwl.common.core.domain.entity.SysUser;
import com.mcwl.common.core.page.TableDataInfo;
import com.mcwl.common.core.redis.RedisCache;
import com.mcwl.common.domain.IdsParam;
import com.mcwl.common.utils.SecurityUtils;
import com.mcwl.common.utils.StringUtils;
import com.mcwl.resource.domain.ModelImage;
import com.mcwl.resource.domain.ModelProduct;
import com.mcwl.resource.domain.ModelVersion;
import com.mcwl.resource.domain.WorkFlowVersion;
import com.mcwl.resource.domain.dto.ModelImagePageRes;
import com.mcwl.resource.domain.request.RequestModel;
import com.mcwl.resource.domain.request.RequestWorkFlow;
import com.mcwl.resource.domain.vo.MallProductVo;
import com.mcwl.resource.domain.vo.ModelImageVo;
import com.mcwl.resource.mapper.MallProductMapper;

import com.mcwl.resource.mapper.ModelVersionMapper;
import com.mcwl.resource.service.ModelService;
import com.mcwl.system.service.ISysUserService;
import lombok.extern.log4j.Log4j;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;


/**模型 业务实现层
 * @Author：ChenYan
 * @Project：McWl
 * @Package：com.mcwl.resource.service.impl
 * @Filename：MallProductServiceImpl
 * @Description TODO
 * @Date：2024/12/30 18:22
 */
@Log4j2
@Service
public class MallProductServiceImpl extends ServiceImpl<MallProductMapper,ModelProduct> implements ModelService {
    @Autowired
    private RedisCache redisCache;

    @Autowired
    private ModelVersionMapper modelVersionMapper;
    @Autowired
    private MallProductMapper postMapper;
    @Autowired
    private ISysUserService sysUserService;


    @Override
    public Page<ModelProduct> selectByUserId(MallProductVo mallProductVo) {

        // 创建分页对象
        Page<ModelProduct> mallProductPage = new Page<>(mallProductVo.getPageNumber(), mallProductVo.getPageSize());

        //获取当前登录人ID
        Long userId = SecurityUtils.getUserId();
        LambdaQueryWrapper<ModelProduct> mallProductLambdaQueryWrapper = new LambdaQueryWrapper<>();

        mallProductLambdaQueryWrapper.eq(ModelProduct::getUserId,userId);
        mallProductLambdaQueryWrapper.eq(ModelProduct::getDelFlag,0);



        // 开始时间和结束时间过滤
        if (mallProductVo.getStartTime() != null && mallProductVo.getEndTime() != null) {
            // 查询开始时间和结束时间之间的商品
            mallProductLambdaQueryWrapper.between(ModelProduct::getCreateTime, mallProductVo.getStartTime(), mallProductVo.getEndTime());
        } else if (mallProductVo.getStartTime() != null) {
            // 只有开始时间，查询大于等于开始时间的商品
            mallProductLambdaQueryWrapper.ge(ModelProduct::getCreateTime, mallProductVo.getStartTime());
        } else if (mallProductVo.getEndTime() != null) {
            // 只有结束时间，查询小于等于结束时间的商品
            mallProductLambdaQueryWrapper.le(ModelProduct::getCreateTime, mallProductVo.getEndTime());
        }

        return postMapper.selectPage(mallProductPage, mallProductLambdaQueryWrapper);
    }

    @Override
    public Page<ModelProduct> pageLike(MallProductVo mallProductVo, List<Long> list) {
        return null;
    }


    /**
     * 分页查询
     *
     * @param imagePageRes 分页参数
     * @return 分页数据
     */
    @Override
    public TableDataInfo listByPage(ModelImagePageRes imagePageRes) {

        Page<ModelProduct> page = new Page<>(imagePageRes.getPageNum(), imagePageRes.getPageSize());
        if (StringUtils.isEmpty(imagePageRes.getOrderByColumn())) {
            imagePageRes.setOrderByColumn("create_time");
        }
        // 设置排序
        boolean isAsc = Objects.equals(imagePageRes.getIsAsc(), "asc");
        OrderItem orderItem = new OrderItem(imagePageRes.getOrderByColumn(), isAsc);
        page.addOrder(orderItem);

        LambdaQueryWrapper<ModelProduct> lqw = new LambdaQueryWrapper<>();
        lqw.eq(imagePageRes.getStatus() != null, ModelProduct::getAuditSatus, imagePageRes.getStatus())
                .eq(imagePageRes.getUserId() != null, ModelProduct::getUserId, imagePageRes.getUserId())
                .eq(imagePageRes.getPageNum() != null, ModelProduct::getUserId, imagePageRes.getUserId())
                .ge(imagePageRes.getStartTime() != null, ModelProduct::getCreateTime, imagePageRes.getStartTime())
                .le(imagePageRes.getEndTime() != null, ModelProduct::getCreateTime, imagePageRes.getEndTime());

        postMapper.selectPage(page, lqw);
        // 获取分页数据
        List<ModelProduct> modelImageList = page.getRecords();

        // Model数据转为ModelPageVo
        List<ModelImageVo> modelImageVoList = new ArrayList<>();
        for (ModelProduct modelImage : modelImageList) {
            ModelImageVo modelImageVo = new ModelImageVo();
            BeanUtil.copyProperties(modelImage, modelImageVo);
            // 获取用户信息
            SysUser sysUser = sysUserService.selectUserById(modelImage.getUserId());
            modelImageVo.setUserId(sysUser.getUserId());
            modelImageVo.setUserName(sysUser.getUserName());
            modelImageVo.setUserAvatar(sysUser.getAvatar());
            modelImageVoList.add(modelImageVo);
        }

        // 封装分页数据
        TableDataInfo rspData = new TableDataInfo();
        rspData.setCode(HttpStatus.SUCCESS);
        rspData.setMsg("查询成功");
        rspData.setRows(modelImageVoList);
        rspData.setTotal(page.getTotal());

        return rspData;
    }

    @Override
    public AjaxResult addModel(RequestModel requestModel) {
        //添加模型表数据
        postMapper.insert(requestModel.getModelProduct());

        log.info("获取到的入参:{}",requestModel.getModelProduct());

        //批量添加版本
        modelVersionMapper.addModelVersion(requestModel.getModelProduct(),requestModel.getModelVersionList());

        return AjaxResult.success("添加成功，等待审核");
    }

    @Override
    public void updaModel(RequestModel requestModel) {
        //修改模版的信息
        ModelProduct modelProduct = requestModel.getModelProduct();
        if (ObjectUtils.isEmpty(modelProduct)){

        }
        if (StringUtils.isNotNull(modelProduct.getId())){
            modelProduct.setAuditSatus(3);
            postMapper.updateModel(requestModel.getModelProduct());
        }

        //修改工作流版本的信息
        if (requestModel.getModelVersionList().size() != 0){

            //批量修改
            for (ModelVersion modelVersion : requestModel.getModelVersionList()) {
                modelVersion.setAuditSatus(3);
                modelVersionMapper.updateById(modelVersion);
            }
        }

    }

}
