package com.mcwl.resource.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.mcwl.common.core.domain.entity.SysUser;
import com.mcwl.common.utils.SecurityUtils;
import com.mcwl.resource.domain.ModelComment;
import com.mcwl.resource.domain.ModelImageComment;
import com.mcwl.resource.domain.dto.ModelImageCommentRes;
import com.mcwl.resource.domain.vo.ModelCommentVo;
import com.mcwl.resource.domain.vo.ModelImageCommentVo;
import com.mcwl.resource.mapper.ModelCommentMapper;
import com.mcwl.resource.mapper.ModelImageCommentMapper;
import com.mcwl.resource.mapper.ModelImageMapper;
import com.mcwl.resource.service.ModelCommentService;
import com.mcwl.system.service.ISysUserService;
import org.jetbrains.annotations.NotNull;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * @Author：ChenYan
 * @Project：McWl
 * @Package：com.mcwl.resource.service.impl
 * @Filename：ModelCommentServiceImpl
 * @Description TODO
 * @Date：2025/1/12 12:03
 */
@Service
public class ModelCommentServiceImpl extends ServiceImpl<ModelCommentMapper, ModelComment> implements ModelCommentService {

    @Autowired
    private  ISysUserService sysUserService;
    @Autowired
    private  ModelCommentMapper modelCommentMapper;

    @Autowired
    private ModelImageMapper modelImageMapper;

    @Override
    public void comment(ModelComment modelComment) {
        Long parentId = modelComment.getParentId();
        ModelComment mic = modelCommentMapper.selectById(parentId);

        if (Objects.nonNull(parentId) && Objects.isNull(mic)) {
            return;
        }
        modelComment.setUserId(SecurityUtils.getUserId());
        modelComment.setCreateBy(SecurityUtils.getUsername());
        modelComment.setUpdateBy(SecurityUtils.getUsername());
        modelComment.setUpdateTime(new Date());
        modelCommentMapper.insert(modelComment);

    }

    @Override
    public List<ModelCommentVo> getComment(Long imageId) {
        List<ModelCommentVo> modelCommentVoList = new ArrayList<>();

        // 查询所有父评论
        LambdaQueryWrapper<ModelComment> lqw = new LambdaQueryWrapper<>();
        lqw.eq(ModelComment::getModelId, imageId)
                .isNull(ModelComment::getParentId)
                .orderByDesc(ModelComment::getCreateTime);
        // 添加父评论
        List<ModelComment> modelCommentList = modelCommentMapper.selectList(lqw);
        for (ModelComment modelComment : modelCommentList) {
            ModelCommentVo modelCommentVo = getModelCommentVo(modelComment);
            modelCommentVoList.add(modelCommentVo);
        }

        return modelCommentVoList;
    }


    /**
     * 构建ModelCommentVo对象
     *
     * @param modelComment 父评论对象
     * @return ModelCommentVo对象
     */
    @NotNull
    private ModelCommentVo getModelCommentVo(ModelComment modelComment) {
        Long userId = modelComment.getUserId();
        SysUser sysUser = sysUserService.selectUserById(userId);

        // 构建ModelCommentVo对象
        ModelCommentVo modelCommentVo = new ModelCommentVo();
        modelCommentVo.setUserId(userId);
        modelCommentVo.setUserName(sysUser.getUserName());
        modelCommentVo.setUserAvatar(sysUser.getAvatar());
        modelCommentVo.setCommentId(modelComment.getId());
        modelCommentVo.setContent(modelComment.getContent());
        // 查询子评论
        modelCommentVo.setContentList(getContentList(modelComment.getId()));
        modelCommentVo.setLikeNum(modelComment.getLikeNum());
        modelCommentVo.setCreateTime(modelComment.getCreateTime());
        return modelCommentVo;
    }


    /**
     * 递归查询子评论
     *
     * @param modelCommentId 父评论id
     * @return List<ModelCommentVo>
     */
    private List<ModelCommentVo> getContentList(Long modelCommentId) {
        List<ModelCommentVo> modelCommentVoList = new ArrayList<>();
        if (Objects.isNull(modelCommentId)) {
            return modelCommentVoList;
        }

        // 查询子评论
        LambdaQueryWrapper<ModelComment> lqw = new LambdaQueryWrapper<ModelComment>()
                .eq(ModelComment::getParentId, modelCommentId)
                .orderByDesc(ModelComment::getCreateTime);

        List<ModelComment> modelCommentList = modelCommentMapper.selectList(lqw);

        for (ModelComment modelComment : modelCommentList) {
            Long userId = modelComment.getUserId();
            SysUser sysUser = sysUserService.selectUserById(userId);
            ModelCommentVo modelCommentVo = new ModelCommentVo();
            modelCommentVo.setUserId(userId);
            modelCommentVo.setUserName(sysUser.getUserName());
            modelCommentVo.setUserAvatar(sysUser.getAvatar());
            modelCommentVo.setCommentId(modelComment.getId());
            modelCommentVo.setContent(modelComment.getContent());
            modelCommentVo.setLikeNum(modelComment.getLikeNum());
            modelCommentVo.setCreateTime(modelComment.getCreateTime());


            modelCommentVoList.add(modelCommentVo);
        }
        return modelCommentVoList;
    }


}
