package com.mcwl.resource.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.mcwl.common.constant.HttpStatus;
import com.mcwl.common.core.domain.entity.SysUser;
import com.mcwl.common.core.page.PageDomain;
import com.mcwl.common.core.page.TableDataInfo;
import com.mcwl.common.utils.SecurityUtils;
import com.mcwl.common.utils.StringUtils;
import com.mcwl.resource.domain.ModelImage;
import com.mcwl.resource.domain.ModelImageComment;
import com.mcwl.resource.domain.dto.ModelImageCommentRes;
import com.mcwl.resource.domain.dto.ModelImagePageRes;
import com.mcwl.resource.domain.dto.ModelImageRes;
import com.mcwl.resource.domain.vo.ModelImageCommentVo;
import com.mcwl.resource.domain.vo.ModelImageVo;
import com.mcwl.resource.mapper.ModelImageCommentMapper;
import com.mcwl.resource.mapper.ModelImageMapper;
import com.mcwl.resource.service.ModelImageService;
import com.mcwl.system.service.ISysUserService;
import lombok.RequiredArgsConstructor;
import org.jetbrains.annotations.NotNull;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;

@Service
@RequiredArgsConstructor
public class ModelImageServiceImpl extends ServiceImpl<ModelImageMapper, ModelImage> implements ModelImageService {

    private final ModelImageCommentMapper modelImageCommentMapper;

    private final ModelImageMapper modelImageMapper;

    private final ISysUserService sysUserService;

    @Override
    public void comment(ModelImageCommentRes modelImageCommentRes) {
        Long parentId = modelImageCommentRes.getParentId();

        if (Objects.nonNull(parentId)) {
            ModelImageComment mic = modelImageCommentMapper.selectById(parentId);
            if (Objects.isNull(mic)) {
                return;
            }
        }

        ModelImageComment modelImageComment = new ModelImageComment();
        BeanUtil.copyProperties(modelImageCommentRes, modelImageComment);
        modelImageComment.setUserId(SecurityUtils.getUserId());
        modelImageComment.setCreateBy(SecurityUtils.getUsername());
        modelImageComment.setCreateTime(new Date());
        modelImageComment.setUpdateBy(SecurityUtils.getUsername());
        modelImageComment.setUpdateTime(new Date());
        modelImageCommentMapper.insert(modelImageComment);

    }

    @Override
    public void updateById(ModelImageRes modelImageRes) {
        if (Objects.isNull(modelImageRes.getId())) {
            return;
        }
        ModelImage modelImage = new ModelImage();
        BeanUtil.copyProperties(modelImageRes, modelImage);
        modelImage.setUpdateBy(SecurityUtils.getUsername());
        modelImage.setUpdateTime(new Date());
        modelImageMapper.updateById(modelImage);
    }

    @Override
    public void publish(ModelImageRes modelImageRes) {

        ModelImage modelImage = new ModelImage();
        BeanUtil.copyProperties(modelImageRes, modelImage);
        modelImage.setUserId(SecurityUtils.getUserId());
        modelImage.setCreateBy(SecurityUtils.getUsername());
        modelImage.setCreateTime(new Date());
        modelImage.setUpdateBy(SecurityUtils.getUsername());
        modelImage.setUpdateTime(new Date());
        modelImage.setStatus(3);
        modelImageMapper.insert(modelImage);


    }


    /**
     * 获取评论
     *
     * @param imageId 图片id
     * @return 评论区
     */
    @Override
    public List<ModelImageCommentVo> getComment(Long imageId) {

        List<ModelImageCommentVo> modelImageCommentVoList = new ArrayList<>();

        // 查询所有父评论
        LambdaQueryWrapper<ModelImageComment> lqw = new LambdaQueryWrapper<>();
        lqw.eq(ModelImageComment::getModelImageId, imageId)
                .isNull(ModelImageComment::getParentId)
                .orderByDesc(ModelImageComment::getCreateTime);
        // 添加父评论
        List<ModelImageComment> modelImageCommentList = modelImageCommentMapper.selectList(lqw);
        for (ModelImageComment modelImageComment : modelImageCommentList) {
            ModelImageCommentVo modelImageCommentVo = getModelImageCommentVo(modelImageComment);
            modelImageCommentVoList.add(modelImageCommentVo);
        }

        return modelImageCommentVoList;
    }


    /**
     * 分页查询
     *
     * @param imagePageRes 分页参数
     * @return 分页数据
     */
    @Override
    public TableDataInfo listByPage(ModelImagePageRes imagePageRes) {

        Page<ModelImage> page = new Page<>(imagePageRes.getPageNum(), imagePageRes.getPageSize());
        if (StringUtils.isEmpty(imagePageRes.getOrderByColumn())) {
            imagePageRes.setOrderByColumn("create_time");
        }
        // 设置排序
        boolean isAsc = Objects.equals(imagePageRes.getIsAsc(), "asc");
        OrderItem orderItem = new OrderItem(imagePageRes.getOrderByColumn(), isAsc);
        page.addOrder(orderItem);

        LambdaQueryWrapper<ModelImage> lqw = new LambdaQueryWrapper<>();
        lqw.eq(imagePageRes.getStatus() != null, ModelImage::getStatus, imagePageRes.getStatus())
                .eq(imagePageRes.getUserId() != null, ModelImage::getUserId, imagePageRes.getUserId())
                .ge(imagePageRes.getStartTime() != null, ModelImage::getCreateTime, imagePageRes.getStartTime())
                .le(imagePageRes.getEndTime() != null, ModelImage::getCreateTime, imagePageRes.getEndTime());

        modelImageMapper.selectPage(page, lqw);
        // 获取分页数据
        List<ModelImage> modelImageList = page.getRecords();

        // ModelImage数据转为ModelImagePageVo
        List<ModelImageVo> modelImageVoList = new ArrayList<>();
        for (ModelImage modelImage : modelImageList) {
            ModelImageVo modelImageVo = new ModelImageVo();
            BeanUtil.copyProperties(modelImage, modelImageVo);
            // 获取用户信息
            SysUser sysUser = sysUserService.selectUserById(modelImage.getUserId());
            modelImageVo.setUserId(sysUser.getUserId());
            modelImageVo.setUserName(sysUser.getUserName());
            modelImageVo.setUserAvatar(sysUser.getAvatar());
            modelImageVoList.add(modelImageVo);
        }

        // 封装分页数据
        TableDataInfo rspData = new TableDataInfo();
        rspData.setCode(HttpStatus.SUCCESS);
        rspData.setMsg("查询成功");
        rspData.setRows(modelImageVoList);
        rspData.setTotal(page.getTotal());

        return rspData;
    }

    /**
     * 构建ModelImageCommentVo对象
     *
     * @param modelImageComment 父评论对象
     * @return ModelImageCommentVo对象
     */
    @NotNull
    private ModelImageCommentVo getModelImageCommentVo(ModelImageComment modelImageComment) {
        Long userId = modelImageComment.getUserId();
        SysUser sysUser = sysUserService.selectUserById(userId);

        // 构建ModelImageCommentVo对象
        ModelImageCommentVo modelImageCommentVo = new ModelImageCommentVo();
        modelImageCommentVo.setUserId(userId);
        modelImageCommentVo.setUserName(sysUser.getUserName());
        modelImageCommentVo.setUserAvatar(sysUser.getAvatar());
        modelImageCommentVo.setCommentId(modelImageComment.getId());
        modelImageCommentVo.setContent(modelImageComment.getContent());
        // 查询子评论
        modelImageCommentVo.setContentList(getContentList(modelImageComment.getId()));
        modelImageCommentVo.setLikeNum(modelImageComment.getLikeNum());
        modelImageCommentVo.setCreateTime(modelImageComment.getCreateTime());
        return modelImageCommentVo;
    }

    /**
     * 递归查询子评论
     *
     * @param modelImageCommentId 父评论id
     * @return List<ModelImageCommentVo>
     */
    private List<ModelImageCommentVo> getContentList(Long modelImageCommentId) {
        List<ModelImageCommentVo> modelImageCommentVoList = new ArrayList<>();
        if (Objects.isNull(modelImageCommentId)) {
            return modelImageCommentVoList;
        }

        // 查询子评论
        LambdaQueryWrapper<ModelImageComment> lqw = new LambdaQueryWrapper<ModelImageComment>()
                .eq(ModelImageComment::getParentId, modelImageCommentId)
                .orderByDesc(ModelImageComment::getCreateTime);

        List<ModelImageComment> modelImageCommentList = modelImageCommentMapper.selectList(lqw);

        for (ModelImageComment modelImageComment : modelImageCommentList) {
            Long userId = modelImageComment.getUserId();
            SysUser sysUser = sysUserService.selectUserById(userId);
            ModelImageCommentVo modelImageCommentVo = new ModelImageCommentVo();
            modelImageCommentVo.setUserId(userId);
            modelImageCommentVo.setUserName(sysUser.getUserName());
            modelImageCommentVo.setUserAvatar(sysUser.getAvatar());
            modelImageCommentVo.setCommentId(modelImageComment.getId());
            modelImageCommentVo.setContent(modelImageComment.getContent());
            modelImageCommentVo.setLikeNum(modelImageComment.getLikeNum());
            modelImageCommentVo.setCreateTime(modelImageComment.getCreateTime());


            modelImageCommentVoList.add(modelImageCommentVo);
        }
        return modelImageCommentVoList;
    }


}
